@@ -35,9 +35,9 @@
 gnutls_protocol_t _gnutls_version_lowest (gnutls_session_t session);
 gnutls_protocol_t _gnutls_version_max (gnutls_session_t session);
 int _gnutls_version_priority (gnutls_session_t session,
-			      gnutls_protocol_t version);
+                              gnutls_protocol_t version);
 int _gnutls_version_is_supported (gnutls_session_t session,
-				  const gnutls_protocol_t version);
+                                  const gnutls_protocol_t version);
 int _gnutls_version_get_major (gnutls_protocol_t ver);
 int _gnutls_version_get_minor (gnutls_protocol_t ver);
 gnutls_protocol_t _gnutls_version_get (int major, int minor);
@@ -56,24 +56,24 @@ const char *_gnutls_x509_mac_to_oid (gnutls_mac_algorithm_t mac);
 
 /* Functions for cipher suites. */
 int _gnutls_supported_ciphersuites (gnutls_session_t session,
-				    cipher_suite_st ** ciphers);
+                                    cipher_suite_st ** ciphers);
 int _gnutls_supported_ciphersuites_sorted (gnutls_session_t session,
-					   cipher_suite_st ** ciphers);
+                                           cipher_suite_st ** ciphers);
 const char *_gnutls_cipher_suite_get_name (cipher_suite_st * algorithm);
 gnutls_cipher_algorithm_t _gnutls_cipher_suite_get_cipher_algo (const
-								cipher_suite_st
-								* algorithm);
+                                                                cipher_suite_st
+                                                                * algorithm);
 gnutls_kx_algorithm_t _gnutls_cipher_suite_get_kx_algo (const cipher_suite_st
-							* algorithm);
+                                                        * algorithm);
 gnutls_mac_algorithm_t _gnutls_cipher_suite_get_mac_algo (const
-							  cipher_suite_st *
-							  algorithm);
+                                                          cipher_suite_st *
+                                                          algorithm);
 gnutls_protocol_t _gnutls_cipher_suite_is_version_supported (const
-							     cipher_suite_st *
-							     algorithm,
-							     gnutls_protocol_t);
+                                                             cipher_suite_st *
+                                                             algorithm,
+                                                             gnutls_protocol_t);
 cipher_suite_st _gnutls_cipher_suite_get_suite_name (cipher_suite_st *
-						     algorithm);
+                                                     algorithm);
 
 /* Functions for ciphers. */
 int _gnutls_cipher_is_block (gnutls_cipher_algorithm_t algorithm);
@@ -89,13 +89,13 @@ int _gnutls_kx_is_ok (gnutls_kx_algorithm_t algorithm);
 
 /* Type to KX mappings. */
 gnutls_kx_algorithm_t _gnutls_map_kx_get_kx (gnutls_credentials_type_t type,
-					     int server);
+                                             int server);
 gnutls_credentials_type_t _gnutls_map_kx_get_cred (gnutls_kx_algorithm_t
-						   algorithm, int server);
+                                                   algorithm, int server);
 
 /* KX to PK mapping. */
 gnutls_pk_algorithm_t _gnutls_map_pk_get_pk (gnutls_kx_algorithm_t
-					     kx_algorithm);
+                                             kx_algorithm);
 gnutls_pk_algorithm_t _gnutls_x509_oid2pk_algorithm (const char *oid);
 const char *_gnutls_x509_pk_to_oid (gnutls_pk_algorithm_t pk);
 
@@ -107,23 +107,23 @@ enum encipher_type _gnutls_kx_encipher_type (gnutls_kx_algorithm_t algorithm);
 /* Functions for sign algorithms. */
 gnutls_sign_algorithm_t _gnutls_x509_oid2sign_algorithm (const char *oid);
 gnutls_sign_algorithm_t _gnutls_x509_pk_to_sign (gnutls_pk_algorithm_t pk,
-						 gnutls_mac_algorithm_t mac);
+                                                 gnutls_mac_algorithm_t mac);
 gnutls_pk_algorithm_t _gnutls_x509_sign_to_pk (gnutls_sign_algorithm_t sign);
 const char *_gnutls_x509_sign_to_oid (gnutls_pk_algorithm_t,
-				      gnutls_mac_algorithm_t mac);
+                                      gnutls_mac_algorithm_t mac);
 gnutls_sign_algorithm_t _gnutls_tls_aid_to_sign (const sign_algorithm_st *
-						 aid);
-sign_algorithm_st _gnutls_sign_to_tls_aid (gnutls_sign_algorithm_t sign);
+                                                 aid);
+const sign_algorithm_st* _gnutls_sign_to_tls_aid (gnutls_sign_algorithm_t sign);
 gnutls_mac_algorithm_t
 _gnutls_sign_get_hash_algorithm (gnutls_sign_algorithm_t);
 gnutls_pk_algorithm_t _gnutls_sign_get_pk_algorithm (gnutls_sign_algorithm_t);
 
 int _gnutls_mac_priority (gnutls_session_t session,
-			  gnutls_mac_algorithm_t algorithm);
+                          gnutls_mac_algorithm_t algorithm);
 int _gnutls_cipher_priority (gnutls_session_t session,
-			     gnutls_cipher_algorithm_t algorithm);
+                             gnutls_cipher_algorithm_t algorithm);
 int _gnutls_kx_priority (gnutls_session_t session,
-			 gnutls_kx_algorithm_t algorithm);
+                         gnutls_kx_algorithm_t algorithm);
 
 unsigned int _gnutls_pk_bits_to_subgroup_bits (unsigned int pk_bits);
 
